<?php

namespace Drupal\basket_paypal\Services;

use Drupal\basket_paypal\PayPal;
use Drupal\Core\Url;
use PayPalCheckoutSdk\Orders\OrdersCreateRequest;

class PayPalJs {

  CONST PAY_PAL_JS_LAYOUT = [
    'vertical'        => 'vertical',
    'horizontal'      => 'horizontal',
  ];

  CONST PAY_PAL_JS_COLORS = [
    'gold'            => 'gold',
    'blue'            => 'blue',
    'silver'          => 'silver',
    'white'           => 'white',
    'black'           => 'black',
  ];

  CONST PAY_PAL_JS_SHAPE = [
    'rect'            => 'rect',
    'pill'            => 'pill',
    'sharp'           => 'sharp',
  ];

  CONST CURRENCY = [
    'AUD',
    'BRL',
    'CAD',
    'CZK',
    'DKK',
    'EUR',
    'HKD',
    'HUF',
    'ILS',
    'JPY',
    'MYR',
    'MXN',
    'TWD',
    'NZD',
    'NOK',
    'PHP',
    'PLN',
    'GBP',
    'SGD',
    'SEK',
    'CHF',
    'THB',
    'USD',
  ];

  protected $container;
  protected $payPal;
  protected $basket;
  protected $config;

  public function __construct(PayPal $pay_pal){
    $this->container = \Drupal::getContainer();
    $this->payPal = $pay_pal;
    $this->basket = $this->container->get('Basket');
    $this->config = \Drupal::config('basket_paypal_js.settings')->get('config');
  }

  public function getDefaultSettings(){
    return [
      'style' => [
        'layout'        => $this->config['style']['layout'] ?? key(static::PAY_PAL_JS_LAYOUT),
        'color'         => $this->config['style']['color'] ?? key(static::PAY_PAL_JS_COLORS),
        'shape'         => $this->config['style']['shape'] ?? key(static::PAY_PAL_JS_SHAPE),
      ],
    ];
  }

  public function getWidgetInfo($data = []){
    $info = $data['config'] ?? [];
    $info = array_replace_recursive($this->getDefaultSettings(), $info);
    return $info;
  }

  public function createOrder($parmentInfo, $params = []) {
    $payment = $this->payPal->load($parmentInfo);
    foreach($parmentInfo as $paramKey => $param){
      if(property_exists($payment, $paramKey)){
        $payment->{$paramKey} = $param;
      }
    }
    $this->payPal->update($payment);

    $_SESSION['basket_paypal_js_last_pay'] = $payment->id;
    $result = [
      'payment'       => clone $payment,
    ];

    $payment->result_url = Url::fromRoute('basket_paypal.pages', ['page_type' => 'result'], ['absolute' => TRUE])->toString();
    $payment->cancel_url = Url::fromRoute('basket_paypal.pages', ['page_type' => 'cancel'], ['absolute' => TRUE])->toString();

    $config = $this->payPal->getConfig();

    $defParams = [
      'intent'          => 'CAPTURE',
      'purchase_units'  => [[
        'reference_id'    => $payment->id,
        'amount'          => [
          'value'           => $payment->amount ?? 0,
          'currency_code'   => $payment->currency ?? 'USD',
        ],
        'is_final_capture'    => TRUE,
      ]],
      'application_context' => [
        'cancel_url'          => $payment->result_url,
        'return_url'          => $payment->result_url,
      ]
    ];
    $params = array_replace_recursive($defParams, $params);

    // Alter
    \Drupal::moduleHandler()->alter('basket_paypal_order_params', $params, $payment, $config);

    $_SESSION['basket_paypal_last_pay'] = $payment->id;

    $client = $this->payPal->client($config);

    $request = new OrdersCreateRequest();
    $request->prefer('return=representation');
    $request->body = $params;
    try {
      // Call API with your client and get a response for your call
      $result['result'] = $client->execute($request);
    }
    catch (\Exception $ex) {
//      dd(@json_decode($ex->getMessage()));
      \Drupal::logger('BasketPayPal')->notice('<pre>'.print_r(@json_decode($ex->getMessage()), TRUE).'</pre>');
    }

    return $result;
  }

  public function getCurrency(){
    $currencyCurrent = $this->basket->currency()->getCurrent();
    $iso = $this->basket->currency()->load($currencyCurrent)->iso;
    if(in_array($iso, static::CURRENCY)){
      return $iso;
    } else {
      return 'USD';
    }
  }

}
